#!/usr/bin/env node
const { Command } = require('commander');
const program = new Command();
const { prompt } = require('enquirer');
const { logError, logSuccess, logInfo } = require("../libs/utils")
const generator = require('generate-password');
const func = require('../libs/function');

const ciqlJSON = require("ciql-json")
const { join } = require("path");
const { existsSync } = require('fs');
const { cwd } = require('process');
const md5 = require('md5')

//Init cmd functions
const init = require('./init/init')

//DB cmd functions
const connect = require('./db/connect')
const disconnect = require('./db/disconnect')
const databases = require('./db/list');
const rm_db = require('./db/rm');
const edit_db = require('./db/edit');
const clear_db = require('./db/clear');

//Code cmd functions
const generate = require('./code/generate')
const update = require('./code/update')
const build = require('./code/build')
const publish = require('./code/publish');

//Todos cmd functions
const add_todo = require('./todo/add')
const list_todo = require('./todo/list')
const check_todo = require('./todo/check');
const rm_todo = require('./todo/rm');
const clear_todo = require('./todo/clear');

//Server cmd functions
const server_rm = require('./server/rm');
const server_ls = require('./server/list');
const server_clear = require('./server/clear');
const server_add = require('./server/add');
const server_edit = require('./server/edit');

//Secure cmd functions
const secure_lock = require('./secure/lock')
const secure_unlock = require('./secure/unlock')

//Info cmd functions
const inspect = require('./info/inspect');
const history = require('./info/history');
const edit_info = require('./info/edit');

//Git cmd function
const git_hook = require('./git/hook')

//set env variables
process.env.CONFIG_PATH = ".ciql"


//hight level commands
const code = program
  .command('code')
  .description("Manage code and API's state");

const db = program
  .command('db')
  .description("Manage databases");

const info = program
  .command('info')
  .description("Get information about project");

const initCmd = program
  .command('init <folder>')
  .description("Init project");

const git = program
  .command('git')
  .description("Manage git repository");

const secure = program
  .command('secure')
  .description("Manage secutity and access of ciql commands");

const server = program
  .command('server')
  .description("Manage servers and deployment");

const todo = program
  .command('todo')
  .description("Manage todo and task");


//For initializing project
initCmd
  .action(async (folder) => {

    //verify if we are inside ciql project
    if (existsSync(".ciql")) {
      logError("Can't initialize ciql project inside ciql project")
      process.exit(0)
    }

    //verify if the folder already exist
    if (existsSync(folder)) {
      logError(`Folder ${folder} already exist`)
      process.exit(0)
    }

    prompt([
      {
        type: 'input',
        name: 'apiname',
        message: 'Project name',
        initial: folder
      },
      {
        type: 'input',
        name: 'description',
        message: 'Description'
      },
      {
        type: 'form',
        name: 'author',
        message: 'Author',
        choices: [
          { name: "name", message: "name" },
          { name: "email", message: "email" },
          { name: "url", message: "url" }
        ]
      }

    ])
      .then(({ apiname, description, author }) => {
        init(apiname.trim(), description, author)
          .then(mes => { logSuccess(mes) })
          .catch(err => { logError(err) })
      })
      .catch((reason) => {
        logError("Broken");
      });

  });

//Connet Api to existed database
db.command('connect [config]')
  .description('Connect to existed database')
  .option('-s, --save <name>', 'Save database configuration')
  .option('-f, --force', 'Force database connection without verification')
  .action(async (config, options) => {
    func.verify(async () => {
      if (func.checkStatus("generated", "init")) {
        let { save, force } = options
        if (config) {
          let pass;

          const configs = ciqlJSON.create(func.readCryptJson(join(cwd(), ".ciql/data/.databases"))).getKeys()
          if (!configs.includes(config)) {
            logError(config + " config not found")
            process.exit(1)
          }
          let { dialect, user, id, host, port, database, pass: filepass } = ciqlJSON.create(func.readCryptJson(join(cwd(), ".ciql/data/.databases"))).getData()[config]

          pass = filepass

          try {

            let connString = { data: `${dialect.toLowerCase()}://${user}:${pass}@${host}:${port}/${database}` }

            connect(connString, force, id)
              .then(mes => {
                let dataPath = join(cwd(), ".ciql", "data", ".databases")
                let data = ciqlJSON
                  .create(func.readCryptJson(dataPath))
                  .set(config, { id, dialect, host, port, user, database, pass })
                  .getData()

                func.writeCryptJson(data, dataPath)
                logSuccess(mes)
              })
              .catch(err => { logError(err) })
              .finally(() => { process.exit(0) })

          } catch (error) {
            logError(error)
          }

        } else {
          prompt([
            {
              type: "select",
              name: "dialect",
              message: "Dialect",
              choices: ["Postgres", "Mysql", "Mariadb"],
            },
            {
              type: 'input',
              name: 'host',
              message: 'Host',
              initial: 'localhost'
            },
            {
              type: 'input',
              name: 'port',
              message: 'Port',
              initial: '5432'
            },
            {
              type: 'input',
              name: 'database',
              message: 'Database'
            },
            {
              type: 'input',
              name: 'user',
              message: 'User'
            },
            {
              type: 'invisible',
              name: 'pass',
              message: 'Password'
            }
          ]).then(({ dialect, user, pass, host, port, database }) => {
            let connString = { data: `${dialect.toLowerCase()}://${user}:${pass}@${host}:${port}/${database}` }

            let identifiant = generator.generate({
              length: 17,
              uppercase: false,
              symbols: false,
              numbers: true
            });

            identifiant = "db-" + identifiant

            connect(connString, force, identifiant)
              .then(mes => {
                logSuccess(mes)
                if (save) {

                  let dataPath = join(cwd(), ".ciql", "data", ".databases")
                  let data = ciqlJSON
                    .create(func.readCryptJson(dataPath))
                    .set(save, { id: identifiant, dialect, host, port, user, database, pass })
                    .getData()

                  func.writeCryptJson(data, dataPath)
                  logSuccess("Configuration saved")
                }
              })
              .catch(err => { logError(err); process.exit(1) })
              .finally(() => { process.exit(0) })
          })
            .catch((reason) => {
              logError("Broken ");
            });
        }
      }
    })
  })

db.command("disconnect")
  .description("Disconnect project to database connected")
  .action(() => {
    func.verify(() => {
      if (func.checkStatus("generated", "init")) {
        disconnect()
          .then(mes => { logSuccess(mes) })
          .catch(err => { logError(err); process.exit(1) })
          .finally(() => { process.exit(0) })
      }
    })
  })

db.command("ls")
  .description("Show all databases information")
  .action(() => {
    func.verify(() => {
      if (func.checkStatus("generated", "built", "init")) {
        databases()
          .then()
          .catch(err => { logError(err); process.exit(1) })
          .finally(() => { process.exit(0) })
      }
    })
  })

db.command("edit <config>")
  .description("Edit db configuration")
  .option('-f, --force', 'Force database editing without connexion verification')
  .action((config, { force }) => {
    func.verify(() => {
      if (func.checkStatus("generated", "init")) {

        const configs = ciqlJSON.create(func.readCryptJson(join(cwd(), ".ciql/data/.databases"))).getKeys()
        if (!configs.includes(config)) {
          logError(config + " config not found")
          process.exit(1)
        }
        let { dialect, user, id, host, port, database, pass: filepass } = ciqlJSON
          .create(func.readCryptJson(join(cwd(), ".ciql/data/.databases")))
          .getData()[config]

        prompt([
          {
            type: "select",
            name: "dialect",
            message: "Dialect",
            choices: ["Postgres", "Mysql", "Mariadb"],
            initial: dialect
          },
          {
            type: 'input',
            name: 'host',
            message: 'Host',
            initial: host
          },
          {
            type: 'input',
            name: 'port',
            message: 'Port',
            initial: port
          },
          {
            type: 'input',
            name: 'database',
            message: 'Database',
            initial: database
          },
          {
            type: 'input',
            name: 'user',
            message: 'User',
            initial: user
          },
          {
            type: 'password',
            name: 'pass',
            message: 'Password',
            initial: filepass
          }
        ]).then(({ dialect, user, pass, host, port, database }) => {

          let connString = { data: `${dialect.toLowerCase()}://${user}:${pass}@${host}:${port}/${database}` }

          edit_db(connString, force)
            .then(mes => {
              logSuccess(mes)

              let dataPath = join(cwd(), ".ciql", "data", ".databases")
              let data = ciqlJSON
                .create(func.readCryptJson(dataPath))
                .set(config, { id, dialect, host, port, user, database, pass })
                .getData()

              func.writeCryptJson(data, dataPath)

              logSuccess("Configuration saved")
            })
            .catch(err => { logError(err); process.exit(1) })
            .finally(() => { process.exit(0) })
        })
          .catch((reason) => {
            logError("Broken : ", reason);
          });
      }
    })
  })

db.command("rm <config>")
  .description("Remove db configuration")
  .action((config) => {
    func.verify(() => {
      if (func.checkStatus("generated", "init")) {
        rm_db(config)
          .then()
          .catch(err => { logError(err); process.exit(1) })
          .finally(() => { process.exit(0) })
      }
    })
  })

db
  .command("clear")
  .description("Delete all saved config")
  .action(() => {
    func.verify(() => {
      if (func.checkStatus("generated", "init")) {
        clear_db()
          .then(mes => { logSuccess("All database's config removed") })
          .catch(err => { logError(err); process.exit(1) })
          .finally(() => { setTimeout(() => { process.exit(0) }, 500) })
      }
    })
  })

//update API structure following schema file
code
  .command('update')
  .description('Update API structure following schema file')
  .action(() => {
    func.verify(() => {
      if (func.checkStatus("init", "generated")) {
        if (func.getStatus() === "init") {
          generate()
            .then(mes => { logSuccess(mes) })
            .catch(err => { logError(err); process.exit(1) })
            .finally(() => { setTimeout(() => { process.exit(0) }, 1000) })
        } else {
          update()
            .then(mes => { logSuccess(mes) })
            .catch(err => { logError(err); process.exit(1) })
            .finally(() => { setTimeout(() => { process.exit(0) }, 1000) })
        }

      }
    })
  });

//update API structure following schema file
code
  .command('build')
  .description('Build project in local file')
  .option('-p, --path <folder>', 'Path where you want to build project')
  .action((options) => {
    func.verify(() => {
      if (func.checkStatus("generated")) {
        let { path } = options
        build(path)
          .then(() => { logSuccess("Project deploy in", path) })
          .catch(err => { logError(err); process.exit(1) })
          .finally(() => { setTimeout(() => { process.exit(0) }, 1000) })
      }
    })
  });

//Deploy code on server
code
  .command('publish <serverConf>')
  .description('Publish code on server')
  .option('-D, --docker', 'Integrate docker file')
  .option('-d, --db <database>', 'Choose another database')
  .action((serverConf, options) => {
    func.verify(async () => {
      if (func.checkStatus("generated")) {
        try {

          let { db: dbConf } = options

          const configs = ciqlJSON.create(func.readCryptJson(join(cwd(), ".ciql/data/.servers"))).getKeys()
          if (!configs.includes(serverConf)) {
            logError(serverConf + " config not found")
            process.exit(1)
          }

          let { servername, host, port, user, db: database, path, cmd, pass } = ciqlJSON
            .create(func.readCryptJson(join(cwd(), ".ciql/data/.servers")))
            .getData()[serverConf]

          let deployedDB = dbConf || database


          const configsD = ciqlJSON.create(func.readCryptJson(join(cwd(), ".ciql/data/.databases"))).getKeys()
          if (!configsD.includes(deployedDB)) {
            logError(deployedDB + "config not found")
            process.exit(1)
          }

          let { dialect, user: dbUser, id: dbId, host: dbHost, port: dbPort, database: dbdatabase, pass: dbPass } = ciqlJSON.create(func.readCryptJson(join(cwd(), ".ciql/data/.databases"))).getData()[deployedDB]

          let connString = { data: `${dialect.toLowerCase()}://${dbUser}:${dbPass}@${dbHost}:${dbPort}/${dbdatabase}` }

          publish({ servername, host, port, user, pass }, { db: deployedDB, dbConn: connString, dbId }, path, cmd)
            .then(mes => {
              logSuccess(mes)
            })
            .catch(err => { logError(err); process.exit(1) })
            .finally(() => { setTimeout(() => { process.exit(0) }, 1000) })

        }
        catch (err) {
          logError("Broken")
        }
      }

    })

  });


//Inspect project
info
  .command('inspect')
  .description('Get informations about project')
  .action(() => {
    func.verify(() => {
      if (func.checkStatus("init", "generated", "built")) {
        inspect()
          .then()
          .catch(err => { logError(err); process.exit(1) })
          .finally(() => { setTimeout(() => { process.exit(0) }, 1000) })
      }
    })
  });

//Inspect project
info
  .command('edit')
  .description('Get informations about project')
  .action(() => {
    func.verify(() => {
      if (func.checkStatus("init", "generated")) {

        const { name, description, author } = ciqlJSON
          .create(func.readCryptJson(join(process.cwd(), '.ciql/data/.info')))
          .extract("{name, description, author}")
          .getData()


        prompt([
          {
            type: 'input',
            name: 'name',
            message: 'Project name',
            initial: name
          },
          {
            type: 'input',
            name: 'description',
            message: 'Description',
            initial: description
          },
          {
            type: 'form',
            name: 'author',
            message: 'Author',
            choices: [
              { name: "name", message: "name", initial: author.name },
              { name: "email", message: "email", initial: author.email },
              { name: "url", message: "url", initial: author.url }
            ]
          }

        ])
          .then(({ name, description, author }) => {
            edit_info(name, description, author)
              .then(mes => { logSuccess(mes) })
              .catch(err => { logError(err); process.exit(1) })
              .finally(() => { setTimeout(() => { process.exit(0) }, 1000) })
          })
          .catch(err => { logError(err); process.exit(1) })
          .finally(() => { setTimeout(() => { process.exit(0) }, 1000) })
      }
    })
  });


//Deployment history
info
  .command('history')
  .description('Get informations about project')
  .action(() => {
    func.verify(() => {
      if (func.checkStatus("generated")) {
        history()
          .then()
          .catch(err => { logError(err); process.exit(1) })
          .finally(() => { setTimeout(() => { process.exit(0) }, 1000) })
      }
    })
  });

git
  .command("hook")
  .description("Enable/Disable hook")
  .action(() => {
    func.verify(() => {
      if (func.checkStatus("generated", "init")) {
        if (func.checkGit()) {
          try {

            const keysHooks = ciqlJSON.create(func.readCryptJson(join(cwd(), ".ciql/data/.hooks"))).remove("iat").getKeys()
            const hooks = ciqlJSON.create(func.readCryptJson(join(cwd(), ".ciql/data/.hooks"))).remove("iat").getData()

            prompt([
              {
                type: "multiselect",
                name: "hooks",
                message: "Choose the hooks you want to active",
                initial: keysHooks.filter(value => hooks[value] === true),
                choices:
                  keysHooks
                    .map((key, i) => ({
                      name: key,
                      value: hooks[key]
                    }))
                ,
                result(names) {
                  return Object.keys(this.map(names));
                }
              }
            ]).then(({ hooks }) => {
              git_hook(hooks)
                .then(mes => { logSuccess(mes) })
                .catch(err => { logError(err); process.exit(1) })
                .finally(() => { process.exit(0) })
            })
              .catch((reason) => { logError("Broken"); });

          } catch (error) {
            logError(error.message || "")
            process.exit(1)
          }
        }
      }
    })
  })


server
  .command("ls")
  .description("show all connected servers information")
  .action(() => {
    func.verify(() => {
      if (func.checkStatus("generated", "init")) {
        server_ls()
          .then()
          .catch(err => { logError(err); process.exit(1) })
          .finally(() => { setTimeout(() => { process.exit(0) }, 500) })
      }
    })
  })

server
  .command("rm <config>")
  .description("Delete a saved config")
  .action((config) => {
    func.verify(() => {
      if (func.checkStatus("generated")) {
        server_rm(config)
          .then()
          .catch(err => { logError(err); process.exit(1) })
          .finally(() => { setTimeout(() => { process.exit(0) }, 500) })
      }
    })
  })

server
  .command("clear")
  .description("Delete all saved config")
  .action(() => {
    func.verify(() => {
      if (func.checkStatus("generated")) {
        server_clear()
          .then(mes => { logSuccess("All server's config removed") })
          .catch(err => { logError(err); process.exit(1) })
          .finally(() => { setTimeout(() => { process.exit(0) }, 500) })
      }
    })
  })

//Add server config
server
  .command('add <config>')
  .description('Add new server configuration')
  .action((config) => {
    func.verify(async () => {
      if (func.checkStatus("generated", "init")) {

        const databasesConfig = ciqlJSON
          .create(func.readCryptJson(join(process.cwd(), '.ciql', 'data', '.databases')))
          .remove('iat')
          .getKeys()

        prompt([
          {
            type: "input",
            name: "host",
            message: "Host",
          },
          {
            type: "input",
            name: "port",
            message: "Port",
            initial: 22
          },
          {
            type: "input",
            name: "user",
            message: "User",
          },
          {
            type: "invisible",
            name: "password",
            message: "Password",
          },
          {
            type: "select",
            name: "db",
            message: "Chose database you want to connected",
            choices: databasesConfig
          },
          {
            type: "input",
            name: "path",
            message: "Path",
            initial: "/var/www"
          },
          {
            type: "input",
            name: "cmd",
            message: "Cmd",
            initial: "npm i"
          }
        ])
          .then(({ host, port, user, password, db, path, cmd }) => {
            server_add(config, host, port, user, password)
              .then(mes => {
                logSuccess(mes)

                let identifiant = generator.generate({
                  length: 17,
                  uppercase: false,
                  symbols: false,
                  numbers: true
                });

                identifiant = "sv-" + identifiant

                let dataPath = join(cwd(), ".ciql", "data", ".servers")

                let data = ciqlJSON
                  .create(func.readCryptJson(dataPath))
                  .set(config, { id: identifiant, servername: config, host, port, user, db, path, cmd, pass: password })
                  .getData()

                func.writeCryptJson(data, dataPath)
                logSuccess("Configuration saved")
              })
              .catch(err => { logError(err); process.exit(1) })
              .finally(() => { setTimeout(() => { process.exit(0) }, 1000) })
          })
          .catch(reason => { logError("Broken") })
      }
    })
  })

//Edit server config
server
  .command('edit <config>')
  .description('Edit server configuration')
  .action((config) => {
    func.verify(async () => {
      if (func.checkStatus("generated")) {

        const databasesConfig = ciqlJSON
          .create(func.readCryptJson(join(process.cwd(), '.ciql', 'data', '.databases')))
          .remove('iat')
          .getKeys()

        const configs = ciqlJSON.create(func.readCryptJson(join(cwd(), ".ciql/data/.servers"))).getKeys()
        if (!configs.includes(config)) {
          logError(config + " config not found")
          process.exit(1)
        }

        const { servername, host, port, user, path, cmd, id, pass } = ciqlJSON
          .create(func.readCryptJson(join(cwd(), ".ciql/data/.servers")))
          .getData()[config]

        prompt([
          {
            type: "input",
            name: "servername",
            message: "Server name",
            initial: servername
          },
          {
            type: "input",
            name: "host",
            message: "Host",
            initial: host
          },
          {
            type: "input",
            name: "port",
            message: "Port",
            initial: port
          },
          {
            type: "input",
            name: "user",
            message: "User",
            initial: user
          },
          {
            type: "password",
            name: "password",
            message: "Password",
            initial: pass
          },
          {
            type: "select",
            name: "db",
            message: "Chose database you want to connected",
            choices: databasesConfig
          },
          {
            type: "input",
            name: "path",
            message: "Path",
            initial: path
          },
          {
            type: "input",
            name: "cmd",
            message: "Cmd",
            initial: cmd
          }
        ])
          .then(({ servername, host, port, user, db, password, path, cmd }) => {
            server_edit(config, host, port, user, password)
              .then(mes => {
                logSuccess(mes)

                let dataPath = join(cwd(), ".ciql", "data", ".servers")

                let data = ciqlJSON
                  .create(func.readCryptJson(dataPath))
                  .set(config, { id, servername, host, port, user, db, path, cmd, pass: password })
                  .getData()

                func.writeCryptJson(data, dataPath)
                logSuccess("Configuration saved")
              })
              .catch(err => { logError(err); process.exit(1) })
              .finally(() => { setTimeout(() => { process.exit(0) }, 1000) })
          })
          .catch(() => { logError("Broken") })
      }
    })
  })


todo
  .command("add")
  .description("add new todo")
  .action(() => {
    func.verify(async () => {
      if (func.checkStatus("generated", "init")) {
        let tasks = []
        let fin = false
        try {
          while (!fin) {
            let { task } = await prompt([
              {
                type: "input",
                name: "task",
                message: "Enter the task (empty to end)",
              }
            ])

            fin = task ? false : true
            task && tasks.push(task)
          }

        } catch (error) {
          logError("Broken ")
          process.exit(1)
        }

        add_todo(tasks)
          .then(mes => { logSuccess(mes) })
          .catch(err => { logError(err); process.exit(1) })
          .finally(() => { fin && setTimeout(() => { process.exit(0) }, 500) })

      }

    })
  })

todo
  .command("check")
  .description("Check todo")
  .action(() => {
    func.verify(() => {
      if (func.checkStatus("generated", "init")) {
        try {

          const keysTasks = ciqlJSON.create(func.readCryptJson(join(cwd(), ".ciql/data/.todos"))).remove("iat").getKeys()
          const tasks = ciqlJSON.create(func.readCryptJson(join(cwd(), ".ciql/data/.todos"))).remove("iat").getData()

          if (keysTasks.length === 0) {
            logError("Any tasks saved")
            process.exit(1)
          }

          prompt([
            {
              type: "multiselect",
              name: "task",
              message: "Select (with space) the ended task",
              choices:
                keysTasks
                  .filter(key => !tasks[key].checked)
                  .map((key, i) => ({
                    name: tasks[key].task,
                    value: key
                  }))
              ,
              result(names) {
                return this.map(names);
              }
            }
          ]).then(({ task }) => {
            check_todo(task)
              .then(mes => { logSuccess(mes) })
              .catch(err => { logError(err); process.exit(1) })
              .finally(() => { process.exit(0) })
          })
            .catch((reason) => { logError("Broken"); });


        } catch (error) {
          logError(error.message || "")
          process.exit(1)
        }
      }
    })
  })

todo
  .command("ls")
  .description("Show all todo")
  .action(() => {
    func.verify(() => {
      if (func.checkStatus("generated", "built", "init")) {
        list_todo()
          .then()
          .catch(err => { logError(err); process.exit(1) })
          .finally(() => { process.exit(0) })
      }
    })
  })

todo
  .command("rm <taskId>")
  .description("Remove todo")
  .action((taskId) => {
    func.verify(() => {
      if (func.checkStatus("generated", "init")) {
        rm_todo(taskId)
          .then(mes => { logSuccess(mes) })
          .catch(err => { logError(err); process.exit(1) })
          .finally(() => { process.exit(0) })
      }
    })
  })

todo
  .command("clear")
  .description("Delete all saved config")
  .action(() => {
    func.verify(() => {
      if (func.checkStatus("generated", "init")) {
        clear_todo()
          .then(mes => { logSuccess("All todo's config removed") })
          .catch(err => { logError(err); process.exit(1) })
          .finally(() => { setTimeout(() => { process.exit(0) }, 500) })
      }
    })
  })

secure
  .command('lock')
  .description('Lock ciql command')
  .option('-p, --password', 'Force password\'s entry')
  .action((options) => {
    func.verify(async () => {
      if (func.checkStatus("generated", "init")) {
        try {
          const { secure: { active, pass: passfile } } = ciqlJSON
            .create(func.readCryptJson(join(process.cwd(), '.ciql/data/.info')))
            .extract("{ secure : { active, pass } }")
            .getData()

          //la fonction n'a jamais été activé
          if (active === false) {
            let { pass, pass2 } = await prompt([
              {
                type: "invisible",
                name: "pass",
                message: "Password",
              },
              {
                type: "invisible",
                name: "pass2",
                message: "Confirm pass",
              }
            ])

            //les mots de passe correspondent
            if (pass === pass2) {

              secure_lock(md5(pass))
                .then(mes => { logSuccess(mes) })
                .catch(err => { logError(err); process.exit(1) })
                .finally(() => { setTimeout(() => { process.exit(0) }, 500) })

            } else {
              logError("Passwords that do not match")
              process.exit(0)
            }
          } else {

            let { password } = options

            //L'option password est sollicitée
            if (password) {

              logInfo("For changing password, enter old password")

              let { oldpass, pass, pass2 } = await prompt([
                {
                  type: "invisible",
                  name: "oldpass",
                  message: "Old password",
                },
                {
                  type: "invisible",
                  name: "pass",
                  message: "New password",
                },
                {
                  type: "invisible",
                  name: "pass2",
                  message: "Confirm password",
                }
              ])

              // l'ancien password corespond
              if (md5(oldpass) === passfile) {
                if (pass === pass2) {

                  secure_lock(md5(pass))
                    .then(mes => { logSuccess(mes) })
                    .catch(err => { logError(err); process.exit(1) })
                    .finally(() => { setTimeout(() => { process.exit(0) }, 500) })

                } else {
                  logError("Passwords that do not match")
                  process.exit(0)
                }
              } else {
                logError("Old password is incorrect")
                process.exit(0)
              }

            } else {

              logInfo("This project has been blocked before")
              logInfo("You must enter the current blocking code or specify --password to change the code")

              let { pass } = await prompt([
                {
                  type: "invisible",
                  name: "pass",
                  message: "Password",
                }])

              if (md5(pass) === passfile) {

                secure_lock(md5(pass))
                  .then(mes => { logSuccess(mes) })
                  .catch(err => { logError(err); process.exit(1) })
                  .finally(() => { setTimeout(() => { process.exit(0) }, 500) })

              } else {
                logError("Password is incorrect")
                process.exit(0)
              }
            }

          }
        } catch (error) {
          logError(error.message || "")
          process.exit(1)
        }

      }
    })
  });

secure
  .command('unlock')
  .description('Unlock ciql command')
  .action(async () => {
    func.verify(async () => {

      if (func.checkStatus("locked")) {

        try {

          const { secure: { pass: passfile } } = ciqlJSON
            .create(func.readCryptJson(join(process.cwd(), '.ciql/data/.info')))
            .extract("{ secure : { pass } }")
            .getData()

          let { pass } = await prompt([
            {
              type: "invisible",
              name: "pass",
              message: "Password",
            }])

          if (md5(pass) === passfile) {

            secure_unlock()
              .then(mes => { logSuccess(mes) })
              .catch(err => { logError(err); process.exit(1) })
              .finally(() => { setTimeout(() => { process.exit(0) }, 500) })

          } else {
            logError("Password is incorrect")
            process.exit(0)
          }
        } catch (error) {
          logError(error.message || "")
          process.exit(1)
        }
      }
    })
  });

program.parse(process.args)
