"use strict"

const ejs = require('ejs')
const fs = require('fs')
const { join } = require('path')
const { privateKey, publicKey, iv, passiv, logError } = require('./utils')
const CiqlJson = require("ciql-json")
const generator = require('generate-password');
const { writeCryptJson } = require('./function')
const { cwd } = require('process')
const md5 = require('md5')

const identifiant = generator.generate({
  length: 17,
  uppercase: false,
  symbols: false,
  numbers: true
});

//WRITE IN .CIQL
exports.writeReleases = async (apiname) => {
  CiqlJson
    .create({})
    .save(join(apiname, process.env.CONFIG_PATH, "/data/releases.json"))
}

exports.writePass = async (apiname, description, author) => {

  fs.writeFileSync(join(apiname, process.env.CONFIG_PATH, 'keys', '.pass'), md5(identifiant), (err) => { err && logError(err.message) })
  fs.writeFileSync(join(apiname, process.env.CONFIG_PATH, 'keys', '.prKey.pem'), privateKey, (err) => { err && logError(err.message) })
  fs.writeFileSync(join(apiname, process.env.CONFIG_PATH, 'keys', '.puKey.pem'), publicKey, (err) => { err && logError(err.message) })
  fs.writeFileSync(join(apiname, process.env.CONFIG_PATH, 'keys', '.iv.key'), iv, (err) => { err && logError(err.message) })
  fs.writeFileSync(join(apiname, process.env.CONFIG_PATH, 'keys', '.passiv.key'), passiv, (err) => { err && logError(err.message) })

  writeCryptJson({}, join(apiname, process.env.CONFIG_PATH, 'data', '.servers'), apiname)
  writeCryptJson({}, join(apiname, process.env.CONFIG_PATH, 'data', '.todos'), apiname)
  writeCryptJson({}, join(apiname, process.env.CONFIG_PATH, 'data', '.databases'), apiname)
  writeCryptJson({}, join(apiname, process.env.CONFIG_PATH, 'data', '.history'), apiname)
  writeCryptJson({
    "pre-commit": false,
    "prepare-commit-msg": false,
    "commit-msg": false,
    "post-commit": false,
    "pre-push": false,
    "pre-rebase": false,
  }, join(apiname, process.env.CONFIG_PATH, 'data', '.hooks'), apiname)
  writeCryptJson({
    id: identifiant,
    name: apiname,
    description: description,
    createdAt: new Date(),
    updatedAt: new Date(),
    node_env: "devlopment",
    db: { connected: false },
    secure: { active: false },
    lastVersion: "1.0.0",
    currentVersion: "1.0.0",
    status: "init",
    author: author
  }, join(apiname, process.env.CONFIG_PATH, 'data', '.info'), apiname)
}

//WRITE ON ROOT
exports.writeIndexRoot = async () => {
  let transFile = await ejs.renderFile(join(__dirname, '../templates/index'));
  fs.writeFile(join('index.js'), transFile, (err) => { if (err) throw err })
}

//WRITE ON DOCS
exports.writeDocsInfo = async (apiname) => {
  let { description, author } = CiqlJson.open(join(cwd(), 'package.json'))
    .extract("{description, author}").getData()
  CiqlJson.create({
    openapi: "3.0.3",
    info: {
      title: apiname,
      description: description,
      version: "1.0.0",
      contact: author
    },
    servers: [{ url: "http://localhost:3983" }]
  }).save(join(cwd(), 'docs', 'info.json'))
}

exports.writeDocsIndex = async (tables) => {
  let transFile = await ejs.renderFile(join(__dirname, "../templates/docs/index"), { tables });
  fs.writeFile(join(cwd(), 'docs', '_index.js'), transFile, (err) => { if (err) throw err })
}


exports.writePackage = async (apiname, description, author) => {

  CiqlJson
    .open(join(__dirname, "../templates/package.json"))
    .set("name", apiname)
    .set("description", description)
    .set("author", author)
    .save(join(apiname, 'package.json'))

}

exports.writeEnv = async (apiname) => {
  let transFile = await ejs.renderFile(join(__dirname, "../templates/env"), { apiname });
  fs.writeFile(join(apiname, '.env'), transFile, (err) => { if (err) throw err })
}

exports.writeReadme = async (apiname) => {
  let transFile = await ejs.renderFile(join(__dirname, "../templates/README"));
  fs.writeFile(join(apiname,  'README.md'), transFile, (err) => { if (err) throw err })
}

exports.writeGitIgnore = async (apiname) => {
  let transFile = await ejs.renderFile(join(__dirname, "../templates/gitignore"));
  fs.writeFile(join(apiname,  '.gitignore'), transFile, (err) => { if (err) throw err })
}

exports.writeSchema = async (apiname) => {
  let transFile = await ejs.renderFile(join(__dirname, "../templates/schema"));
  fs.writeFile(join(apiname, 'schema.js'), transFile, (err) => { if (err) throw err })
}

//WRITE IN SERVER
exports.writeApp = async () => {
  let transFile = await ejs.renderFile(join(__dirname, "../templates//server/app"));
  fs.writeFile(join('server', 'app.js'), transFile, (err) => { if (err) throw err })
}

exports.writeGbRoute = async (apiname, tables) => {
  let transFile = await ejs.renderFile(join(__dirname, "../templates/server/globalRoutes"), {
    tables: tables, apiname: apiname
  });
  fs.writeFile(join('server', '_globalRoutes.js'), transFile, (err) => { if (err) throw err })
}

exports.writeIndex = async (apiname) => {
  let transFile = await ejs.renderFile(join(__dirname, "../templates/server/index"), { apiname: apiname });
  fs.writeFile(join('server', 'index.js'), transFile, (err) => { if (err) throw err })
}

//WRITE IN DB

exports.writeIndexDB = async (schema, tables) => {
  let transFile = await ejs.renderFile(join(__dirname, "../templates/db/index"), { tables, schema });
  fs.writeFile(join('db', '_index.js'), transFile, (err) => { if (err) throw err })
}

exports.writeRelations = async (schema, tables) => {
  let transFile = await ejs.renderFile(join(__dirname, "../templates/db/relations"), { schema: schema, tables: tables });
  fs.writeFile(join('db', '_relations.js'), transFile, (err) => { if (err) throw err })
}

//WRITE IN PUBLIC

exports.writeIco = async function writeIco() {
  let transFile = await ejs.renderFile(join(__dirname, "../templates/public/favicon.ico"));
  fs.writeFile(join('public', 'favicon.ico'), transFile, (err) => { if (err) throw err })
}

// -----------------------------------------------------------------------------
//WRITE IN TABLE

exports.writeApiDocs = async (schema, table) => {
  let attrs = schema[table]["attrs"]
  let attrTables = Object.keys(attrs)
  let required = attrTables.filter(at => attrs[at]['allowNull'] === false)

  let props = CiqlJson.create({})
  attrTables.forEach(at => {
    props.set(at, { type: "string" })
  })

  let properties = props.getData()

  CiqlJson.create({})
    .set(`/${table}`, {
      get: {
        operationId: `get${table}s`,
        tags: [table],
        description: `"Returns all ${table}s`,
        responses: {
          200: { description: `List of ${table}s` },
          403: { description: `Unauthorized` },
          500: { description: `Internal Error` }
        }
      },
      post: {
        operationId: `add${table}`,
        tags: [table],
        description: `Add new ${table}`,
        requestBody: {
          content: {
            "application/x-www-form-urlencoded": {
              schema: {
                type: "object",
                properties: properties,
                required: required
              }
            }
          }
        },
        responses: {
          201: { description: "table created" },
          403: { description: `Unauthorized` },
          500: { description: `Internal Error` }
        }
      },
      parameters: [
        {
          in: "header",
          name: "x-access-token",
          schema: { type: "string" }
        }
      ]
    })
    .set(`/${table}/{id}`, {
      get: {
        operationId: `get${table}`,
        tags: [table],
        description: `Returns specific ${table} designated by id`,
        responses: {
          200: { description: `${table} basic information` },
          404: { description: `${table} not found` },
          403: { description: `Unauthorized` },
          500: { description: `Internal Error` }
        }
      },
      put: {
        operationId: `update${table}`,
        tags: [table],
        description: `Update ${table}`,
        requestBody: {
          content: {
            "application/x-www-form-urlencoded": {
              schema: {
                type: "object",
                properties: properties,
                required: required
              }
            }
          }
        },
        responses: {
          201: { description: `${table} updated` },
          404: { description: `${table} not found` },
          403: { description: `Unauthorized` },
          500: { description: `Internal Error` }
        }
      },
      delete: {
        operationId: `delete${table}`,
        tags: [table],
        description: `Delete specific ${table} designated by id`,
        responses: {
          200: {},
          404: { description: `${table} not found` },
          403: { description: `Unauthorized` },
          500: { description: `Internal Error` }
        }
      },
      parameters: [
        {
          in: "path",
          name: "id",
          required: true,
          schema: { type: "integer" }
        },
        {
          in: "header",
          name: "x-access-token",
          schema: { type: "string" }
        }
      ]
    })
    .save(join('api', table, 'docs.json'))
}

exports.writeRoute = async (table) => {
  let transFile = await ejs.renderFile(join(__dirname, "../templates/api/routes"), { table: table });
  fs.writeFile(join('api', table, 'routes.js'), transFile, (err) => { if (err) throw err })
}

exports.writeTest = async (table) => {
  let transFile = await ejs.renderFile(join(__dirname, "../templates/api/test"), { table: table });
  fs.writeFile(join('api', table, 'test.js'), transFile, (err) => { if (err) throw err })
}